package MT::Plugin::QuickEntry;
use strict;
use MT;
@MT::Plugin::QuickEntry::ISA = qw( MT::Plugin );

use MT::Util qw( offset_time_list format_ts );

our $VERSION = '0.2';

my $plugin = new MT::Plugin::QuickEntry ( {
    name => 'QuickEntry',
    id   => 'QuickEntry',
    key  => 'quickentry',
    description => '<__trans phrase="Post entry from Dashboard Widget.">',
    author_name => 'Junnama Noda',
    author_link => 'http://junnama.alfasado.net/online/',
    version => $VERSION,
    l10n_class => 'QuickEntry::L10N',
} );

MT->add_plugin( $plugin );

sub init_registry {
    my $plugin = shift;
    $plugin->registry( {
        widgets => {
            quickentry => {
                label    => 'Quick Entry',
                plugin   => $plugin,
                template => 'quickentry.tmpl',
                singular => 1,
                # set      => 'sidebar',
                # permission => 'create_post',
                condition => sub {
                    my ( $page, $scope ) = @_;
                    return $page eq 'dashboard';
                },
                code => \&_quickentry,
            },
        },
        applications => {
            cms => {
                methods => {
                    quickentry_post =>
                        'MT::Plugin::QuickEntry::_quickentry_post',
                },
            },
        },
        callbacks => {
            'MT::App::CMS::template_param.asset_insert'
                => \&_asset_insert,
        },
   } );
}

sub _quickentry {
    my ( $app, $tmpl, $param ) = @_;
    my $user = $app->user;
    my $blog_view;
    my $blod_exist;
    if ( $app->blog ) {
        $param->{ can_publish } = _is_user_can( $app->blog, 'publish_post' );
        $param->{ can_upload }  = _is_user_can( $app->blog, 'upload' );
        if ( $app->blog->class eq 'blog' ) {
            $blog_view = 1;
            $param->{ blog_view } = 1;
            $blod_exist = 1;
        }
    }
    if (! $blog_view ) {
        my @blog_loop;
        my @weblogs;
        if ( $app->blog ) {
            @weblogs = MT::Blog->load( { parent_id => $app->blog->id } );
        } else {
            @weblogs = MT::Blog->load();
        }
        my $can_publish;
        my $can_upload;
        my @can_publish_ids;
        my @can_upload_ids;
        for my $weblog ( @weblogs ) {
            if ( _is_user_can( $weblog, 'create_post' ) ) {
                $blod_exist = 1;
                $can_publish = _is_user_can( $weblog, 'publish_post' ) if (! $can_publish );
                $can_upload  = _is_user_can( $weblog, 'upload' ) if (! $can_upload );
                if ( _is_user_can( $weblog, 'publish_post' ) ) {
                    push ( @can_publish_ids, $weblog->id );
                }
                if ( _is_user_can( $weblog, 'upload' ) ) {
                    push ( @can_upload_ids, $weblog->id );
                }
                push @blog_loop,
                    {
                        weblog_name => $weblog->name,
                        weblog_id => $weblog->id,
                    };
            }
        }
        $param->{ can_publish } = $can_publish;
        $param->{ can_upload }  = $can_upload;
        $param->{ blog_loop }   = \@blog_loop;
        $param->{ can_publish_ids } = join( ',', @can_publish_ids );
        $param->{ can_upload_ids }  = join( ',', @can_upload_ids );
    }
    $param->{ blod_exist }  = $blod_exist;
}

sub _asset_insert {
    my ( $cb, $app, $param, $tmpl ) = @_;
    if ( $app->param( 'edit_field' ) eq 'quickentry-text' ) {
        my $pointer_field = $tmpl->getElementById( 'insert_script' );
        $pointer_field->innerHTML( "window.parent.custom_insertHTML( '<mt:var name=\"upload_html\" escape=\"js\">', 'quickentry-text' );" );
    }
}

sub _quickentry_post {
    my $app = MT->instance();
    $app->validate_magic() or $app->trans_error( 'Invalid request.' );
    if (! _is_user_can( $app->blog, 'create_post' ) ) {
        return $app->trans_error( 'Permission denied.' );
    }
    my $blog = $app->blog;
    my $user = $app->user;
    require MT::Entry;
    my $entry = MT::Entry->new;
    $entry->blog_id( $blog->id );
    $entry->author_id( $user->id );
    $entry->created_by( $user->id );
    $entry->modified_by( $user->id );
    my $status = $app->param( 'status' );
    if ( $status !~ /^[1-2]{1}$/ ) {
        return $app->trans_error( 'Invalid request.' );
    }
    if ( (! _is_user_can( $app->blog, 'publish_post' ) ) && ( $status == MT::Entry::RELEASE() ) ) {
        return $app->trans_error( 'Permission denied.' );
    }
    $entry->status( $status );
    $entry->allow_comments( $blog->allow_comments_default );
    $entry->allow_pings( $blog->allow_pings_default );
    $entry->class( 'entry' );
    my @tl = offset_time_list( time, $blog );
    my $ts = sprintf "%04d%02d%02d%02d%02d%02d", $tl[5]+1900, $tl[4]+1, @tl[3,2,1,0];
    $entry->authored_on( $ts );
    if ( $entry->allow_pings ) {
        $entry->atom_id( $entry->make_atom_id() );
    }
    my $title = $app->param( 'title' );
    my $text  = $app->param( 'text' );
    $entry->title( $title );
    $entry->text( $text );
    $entry->convert_breaks( '__default__' );
    my $delim = $user->entry_prefs->{ tag_delim } || ',';
    $delim = quotemeta( $delim );
    if ( my $tags  = $app->param( 'tags' ) ) {
        my @t = split( /$delim/, $tags );
        $entry->set_tags( @t );
    }
    $app->run_callbacks( 'cms_pre_save.entry', $app, $entry, undef );
    $entry->save or die $entry->errstr;
    $app->run_callbacks( 'cms_post_save.entry', $app, $entry, $entry );
    my $view_link;
    if ( $entry->status == MT::Entry::RELEASE() ) {
        $app->rebuild_entry( Entry => $entry,
                             BuildDependencies => 1 );
        $view_link = '<a target="_blank" href="' . $entry->permalink . '">' . $app->translate( 'View' ) . '</a>';
    }
    my $edit_uri = $app->base . $app->uri( mode => 'view',
                                           args => { _type => 'entry',
                                                     id => $entry->id,
                                                     blog_id => $blog->id, } );
    my $edit_link = '<a href="' . $edit_uri . '">' . $app->translate( 'Edit' ) . '</a>';
    my $return_str = $app->translate( 'This entry has been saved.' );
    $return_str .= ' ' . $view_link if $view_link;
    $return_str .= ' ' . $edit_link;
    return '<div>' . $return_str . '</div>';
}

sub _is_user_can {
    my ( $blog, $permission ) = @_;
    my $app = MT->instance();
    my $user = $app->user;
    $permission = 'can_' . $permission;
    my $perm = $user->is_superuser;
    unless ( $perm ) {
        if ( $blog ) {
            $perm = $user->permissions( $blog->id )->$permission;
        }
    }
    return $perm;
}

1;